#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 09:40:28 +0300 (Fri, 12 May 2006) $
#$Revision: 1056 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/CCP4Map.pm $
#-----------------------------------------------------------------------
# Reading header information from CCP4 binary map files
#-----------------------------------------------------------------------

package CCP4Map;

use strict;
require Exporter;
@CCP4Map::ISA = qw(Exporter);
@CCP4Map::EXPORT = qw( readRawHeader readMapHeader readHashHeaders readArrayHeaders );

@CCP4Map::header_fields = qw(
   columnNr
   rowNr
   sectionNr
   mode
   columnNrStart
   rowNrStart
   sectionNrStart
   xIntervals
   yIntervals
   zIntervals
   cellA
   cellB
   cellC
   cellAlpha
   cellBeta
   cellGamma
   mapCol
   mapRow
   mapSec
   minDensity
   maxDensity
   meanDensity
   spaceGroup
   symmetryBytes
   skewFlag
   skewMatrix
   skewTranslation
   mapSignature
   machineStamp
   rmsDeviation
   labelNr
   labels
   symmetryOperators
);


my $sizeof_longword = 4;
my $header_size = 256 * $sizeof_longword;
my $labelSize = 80;
my $symmetryRecordSize = 80;


sub readRawHeader
{
    my ( $filename, $map_header ) = @_;
    my $bin_header = "";

    open(MAPFILE, $filename) or die("Can't open $filename: $!");
    read(MAPFILE, $bin_header, $header_size);

    my @map_header = unpack( "L10f6L3f3L15x60a4LfLa800", $bin_header );
    if( !defined $map_header[37] or $map_header[37] ne "MAP " ) {
        close MAPFILE;
        die("file '$filename' is not a CCP4 map\n");
    }

    my $symmetryBytes = $map_header[23];
    my $symops = "";
    read(MAPFILE, $symops, $symmetryBytes);
    push(@map_header, $symops);
    
    close(MAPFILE);

    # transform skew operators
    
    my @skewTranslation = @map_header[34..36];
    my @skewMatrix = @map_header[25..33];
    splice(@map_header, 34, 3, [@skewTranslation] );
    splice(@map_header, 25, 9, [@skewMatrix] );

    return @map_header if not defined $map_header;

    if( ref $map_header eq "ARRAY" ) {
        $map_header = \@map_header;
    } elsif( ref $map_header eq "HASH" ) {
        my $field = 0;
        %$map_header = map {$CCP4Map::header_fields[$field++], $_} @map_header;
    } else {
        die("Need array or hash ref as a second argument for readRawHeader");
    }
}

sub readMapHeader
{

    my ( $filename, $map_header ) = @_;
    my @map_header = &readRawHeader( $filename );

    # transform symmetry operators
    
    my $symmOps = $map_header[-1];
    my $symmetryBytes = $map_header[23];
    $map_header[-1] = [];
    my $symmNr = $symmetryBytes / $symmetryRecordSize;
    my $symmCnt;
    for( $symmCnt = 0; $symmCnt < $symmNr; $symmCnt++ ) {
        push( @{$map_header[-1]}, substr($symmOps, 0, 80));
        $map_header[-1][-1] =~ s/^ *| *$//g;
        substr($symmOps, 0) = substr($symmOps, 80);
    }

    # transform labels
    
    my ($labelNr, $labels) = @map_header[-3..-2];
    $map_header[-2] = [];

    my $labelCnt; 
    for( $labelCnt = 0; $labelCnt < $labelNr; $labelCnt++ ) {
        push( @{$map_header[-2]}, substr($labels, 0, 80));
        $map_header[-2][-1] =~ s/^ *| *$//g;
        substr($labels, 0) = substr($labels, 80);
    }

    return @map_header if not defined $map_header;

    if( ref $map_header eq "ARRAY" ) {
        $map_header = \@map_header;
    } elsif( ref $map_header eq "HASH" ) {
        my $field = 0;
        %$map_header = map {$CCP4Map::header_fields[$field++], $_} @map_header;
    } else {
        die("Need array or hash ref as a second argument for readMapHeader");
    }
}

sub readHashHeaders
{
    my %header;
    my @ret = ();
    foreach (@_) {
        readMapHeader($_, \%header);
        $header{name} = $_;
        push( @ret, { %header } );
    }
    @ret;
}

sub readArrayHeaders
{
    my @header;
    my @ret = ();
    foreach (@_) {
        readMapHeader($_, \@header);
        unshift( @header, $_ );
        push( @ret, [ @header ] );
    }
    @ret;
}
