#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 09:40:28 +0300 (Fri, 12 May 2006) $
#$Revision: 1056 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/Denzo.pm $
#-----------------------------------------------------------------------
# Read and parse DENZO related files
#-----------------------------------------------------------------------

package Denzo;

use strict;
require Exporter;
@Denzo::ISA = qw(Exporter);
@Denzo::EXPORT = qw( loadScript );

$Denzo::number = 
  '[-+]? (?:[0-9]+(?:\.[0-9]*)? | (?:\.[0-9]+)) (?:[eE][-+]?[0-9]+)?';

my %fields = (

 "spot\\s+elliptical\\s+($Denzo::number)\\s+($Denzo::number)"
     => sub { "spot_size", $1, $2 },

 "background\\s+elliptical\\s+($Denzo::number)\\s+($Denzo::number)"
     => sub { "bg_size", $1, $2 },

 "box\\s+($Denzo::number)\\s+($Denzo::number)" 
     => sub { "box_size", $1, $2 },

 "polarisation\\s+($Denzo::number)"
     => sub { "polarisation", $1 },

 "profile\\s+fitting\\s+radius\\s+($Denzo::number)"
     => sub { "profile_radius", $1 },

 "overlap\\s+(\\w+)" 
     => sub { "overlap_type", $1 },

 "resolution\\s+limits\\s+($Denzo::number)(?:\\s+($Denzo::number))?"
     => sub { "resolution", $1, defined $2 ? $2 : () },

 "space\\s+group\\s+([^\\s]+)"
     => sub { "spacegroup", $1 },

 "mosaicity\\s+($Denzo::number)"
     => sub { "mosaicity", $1 },

 "unit\\s+cell\\s+($Denzo::number)\\s+($Denzo::number)\\s+($Denzo::number)".
             "\\s+($Denzo::number)\\s+($Denzo::number)\\s+($Denzo::number)"
     => sub { "cell", $1, $2, $3, $4, $5, $6 },

 "crystal\\s+rotx\\s+($Denzo::number)\\s+".
            "roty\\s+($Denzo::number)\\s+rotz\\s+($Denzo::number)"
     => sub { "crystal_orientation", $1, $2, $3 },

 "detector\\s+rotx\\s+($Denzo::number)\\s+".
             "roty\\s+($Denzo::number)\\s+rotz\\s+($Denzo::number)" 
     => sub { "detector_orientation", $1, $2, $3 },

 "distance\\s+($Denzo::number)"
     => sub { "distance", $1 },

 "wavelength\\s+($Denzo::number)"
     => sub { "wavelength", $1 },

 "x\\s+beam\\s+($Denzo::number)\\s+y\\s+beam\\s+($Denzo::number)"
     => sub { "center", $1, $2 },

 "film\\s+rotation\\s+($Denzo::number)"
     => sub { "film_rotation", $1 },

 "radial\\s+offset\\s+($Denzo::number)\\s+angular\\s+offset\\s+($Denzo::number)" 
     => sub { "detector_offset", $1, $2 },

 "crossfire\\s+y\\s+($Denzo::number)\\s+x\\s+($Denzo::number)".
              "\\s+xy\\s+($Denzo::number)"
     => sub { "crossfire", $2, $1, $3 },

); # %fields

sub loadScript
{
    my $filename = shift;
    my %items = @_;

    open(MATRIX, $filename) or die("could not open '$filename': $!");
    while(<MATRIX>) {
    	if( /^\s*\[.*\]\s*$/ ) {
    	    if( defined $items{comments} ) {
    		s/\r*$//;
    		push(@{$items{comments}}, $_);
    	    }
    	} else {
    	    my ($pattern, $action);
    	    while ( ($pattern, $action) = each %fields ) {
    	        setItem( \%items, &$action ) if /$pattern/ix;
    	    }
        }
    } # while(<MATRIX>)
    close(MATRIX);

} # loadScript

sub setItem
{
    my $hash = shift;
    my $key = shift;
    my @val = @_;

    if( defined $hash->{$key} ) {
       CASE: for( ref($hash->{$key}) ) {
           if( /ARRAY/ ) {
               @{$hash->{$key}} = @val unless @{$hash->{$key}};
               last
           }
           if( /SCALAR/ ) {
               ${$hash->{$key}} = $val[0] if ${$hash->{$key}} eq "" or
                                             not defined ${$hash->{$key}};
               last
           }
	   die("Hash for Denzo::readScript ".
               "must contain only scalar or array refs\n");
       }
    }
}
