#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 09:40:28 +0300 (Fri, 12 May 2006) $
#$Revision: 1056 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/FFTGrid.pm $
#-----------------------------------------------------------------------
# Generate acceptable grid for crystallographic FFT routines
#-----------------------------------------------------------------------

package FFTGrid;

use strict;
require Exporter;
@FFTGrid::ISA = qw(Exporter);
@FFTGrid::EXPORT = qw(choose_grid);

use Primes;

# For all space-groups, NX,NY,NZ must have no prime factors greater
# than 19, and NX must be greater than 2 HMAX + 1, etc.  Below, `n' is
# an integer, so `2n' means the number must be even.

# 	  Space         section
# 	  group  number  axis
# 	  P1       1       y   NZ=2n
# 	  P-1      2       y   NY=2n, NZ=2n, NX=2n,
# 	  P2       3       y   NY=2n, NZ=2n, NX=2n,   must use FFTSP P1
# 	  P21      4       y   NY=4n, NZ=2n,
# 	  P2/M    10       y   NX=2n, NY=8n, NZ=2n,
# 	  P222    16       y   NX=2n, NY=2n, NZ=2n,
# 	  P2221   17       y   NX=2n, NY=2n, NZ=2n,
# 	  P21212  18       y   NY=4n, NZ=2n,
# 	  P21212a 1018     y   NY=4n, NZ=2n,
# 	  P212121 19       z   NY=2n, NZ=4n,
# 	  C2221   20       y   NX=4n, NY=4n, NZ=4n,
# 	  C222    21       y   NX=4n, NY=4n, NZ=4n,
# 	  I222    23       y   NX=4n, NY=4n, NX=4n,
# 	  PMMM    47       z   NX=8n, NY=8n, NZ=8n,
#    P41212/P43212  92,96  z   NX=2n, NY=NX, NZ=8n,
#    P31/P32    144,145    z   NX=6n, NY=6n, NZ=6n
#    R3          146       z   NX=6n, NY=6n, NZ=6n
#    P3121/P3221 152,154   z   NX=6n, NY=6n, NZ=6n
#    P61/P65 169/170       z   NX=6n, NY=6n, NZ=12n

# grid = cell * resolution / 3

# smallest number that satisfies requirements for any group:

my $smallest_grid = 24;

sub choose_grid
{
    my $sampling = shift;
    my @cell = @_;
    my @grid = ();

    my $cell;
    local @Primes::primes = (2,3,5,7,11,13,17,19);

    foreach $cell (@cell) {
        my $min_grid = int($cell / $sampling + 0.5);
        push(@grid, $smallest_grid) and next if $smallest_grid > $min_grid;

        my $coef = int($min_grid / $smallest_grid + 1);
	use integer;
        while(not suitable($coef) and $coef < 1000) { $coef += 1 }
	die("Could not find grid") if $coef >= 1000;
	push(@grid, $smallest_grid * $coef );
    }
    return @grid;
}

sub suitable
{
    eval { &factorise };
    return $@ ? 0 : 1;
}

1;
