#!/usr/bin/perl
#------------------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-03 13:16:28 +0300 (Wed, 03 May 2006) $ 
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/aux/GetOptions.pm $
#$Revision: 988 $
#------------------------------------------------------------------------

package GetOptions;

use strict;
require Exporter;
@GetOptions::ISA = qw(Exporter);
@GetOptions::EXPORT = qw( getOptions getMandatoryValue getOptionalValue
                          usage optionUsage );

use File::Basename;

## $GetOptions::options = undef;
%GetOptions::option_desc = ();

my $float = '[-+]? (\d+(\.\d*)? | (\.\d+)) ([eE][-+]?\d+)?';
my $integer = '[-+]? \d+';

my %OptionTypes = (
   "integer"   => [ "^\\s* ($integer\\s*)  \$",
		    "one integer argument" ],
   "float"     => [ "^\\s* ($float\\s*)    \$",
		    "one floating point argument" ],
   "\@integer" => [ "^\\s* ($integer\\s*)+ \$",
		    "one or several integer arguments" ],
   "\@float"   => [ "^\\s* ($float\\s*)+   \$",
		    "one or several floating point arguments" ],
);

my %Types = ();

sub getOptions
{
   my @unused_ARGV = ();
   my ($options, $defaultAction, $unknownOptionAction) = @_;

   ${$options}{"--help,--usage"} = 
       [ sub { usage(); exit }, "print a short usage message (this message)" ]
           unless defined ${$options}{"--help,--usage"} or 
                  defined ${$options}{"--help"} or
                  defined ${$options}{"--usage"};

   %GetOptions::option_desc = %$options;

   &splitOptions($options);

   option: while( @ARGV > 0 ) {
      if( $ARGV[0] =~ /^@/ ) { 
          splice( @ARGV, 0, 1, &interpolateFile( $ARGV[0] ) );
          redo option;
      }
      if( $ARGV[0] =~ /^-/ and $ARGV[0] ne "-" ) {
         shift @ARGV and last option if $ARGV[0] eq "--";
         if( $ARGV[0] =~ /=/ ) { splice( @ARGV, 0, 1, split(/=/,$ARGV[0],2) ) }
         my $opt_var = $options->{$ARGV[0]};
         if( !defined $opt_var && length($ARGV[0]) > 2 ) {
             if( $ARGV[0] !~ /^--/ ) {
                splice(@ARGV, 0, 1, map {"-$_"} split(//,substr($ARGV[0],1)) );
                redo option;
             } else {
                $opt_var = &guessLongOption($ARGV[0], $options);
             }
         }
         OPT_VAR: for ($opt_var) { if( $opt_var ) {
             if( ! ref($opt_var) ) {
                 $ARGV[0] = $opt_var;
                 redo option;
             }
             SWITCH: for (ref($opt_var)) {
                if( /CODE/   ) { &{$opt_var}; last }
                if( /SCALAR/ ) { 
                    if( $Types{$ARGV[0]} and $Types{$ARGV[0]} =~ /bool/ ) {
	        	${$opt_var} = ($ARGV[0] =~ /^--no-|^-\w-$/) ? 0 : 1;
                    } else {
	        	${$opt_var} = &getMandatoryValue;
	            }
	            last SWITCH;
	        }
	        if( /ARRAY/ ) {
            	    if( @$opt_var > 0 and ref($$opt_var[0]) ) {
            		$opt_var = $$opt_var[0];
	    		redo OPT_VAR;
            	    } else { 
            		push( @$opt_var, &getMandatoryValue );
	    	    }
                    last SWITCH;
                }
                if( /HASH/ ) {
                    my $option = $ARGV[0]; # in case Perl will decide
                			   # to evaluate right side
                			   # before left side
                    $opt_var->{$option} = &getOptionalValue;
                    last SWITCH;
                }
             } # SWITCH
	     shift @ARGV;
         } else {
             defined $unknownOptionAction ?
                 &{$unknownOptionAction} :
                 die "Unknown option $ARGV[0] for $0";
         }} # OPT_VAR
      } else {
          if( $defaultAction ) {
             &{$defaultAction} ;
          } else {
             push( @unused_ARGV, shift(@ARGV) );
          }
      }
   } 

   if( $defaultAction ) {
       while( @ARGV > 0) { 
           &{$defaultAction} ;
       }
   } else {
       @ARGV = ( @ARGV, @unused_ARGV );
   }
}

#-----------------------------------------------------------------------------

sub splitOptions
{
    my $options = shift;
    my $opt;

    foreach $opt (keys %$options) {
        if( $opt =~ /[,:]/ ) {
	    my($option_list, $type) = split(":", $opt);
            my @synonims = split(",", $option_list);
            foreach (@synonims) {
                $$options{$_} = $$options{$opt};
                $Types{$_} = $type;
                if( defined $type and $type =~ /bool/ ) {
                    s/^--(?!no-)/--no-/;
                    s/^-(\w)$/-$1-/;
                    $$options{$_} = $$options{$opt};
		    $Types{$_} = $type;
	        }
            }
            delete $$options{$opt};
        }
    }
}

#------------------------------------------------------------------------------

sub longest 
{
    my $longest = 0;
    foreach (@_) {
	$longest = length unless $longest > length;
    }
    return $longest;
}

sub optionUsage
{
    my $option_hash = (keys(%$_) > 0) ? %$_ : \%GetOptions::option_desc;
    my @options = sort keys %$option_hash;
    my @names = @options;
    map {s/:.*$//; s/,/, /g} @names;
    my $len = longest(@names);
    my $opt;
    foreach $opt (@options) {
        my $opt_var = $$option_hash{$opt};
        $opt = shift(@names);
        if ( ref($opt_var) eq "ARRAY" and defined $$opt_var[1] ) {
    	   printf " %-${len}s  %s\n", $opt, $$opt_var[1];
        }
    }
}

my $license = <<EOT;

This script is free software; you are welcome to use and distribute it
under terms of GPL. See '@{[ basename($0) ]} --license' for details.
EOT

sub usage
{
    open( SCRIPT, $0 ) or die( "could not open file $0: $!" );
    while( <SCRIPT> ) { s/^\s*#\*?\*?// and print if /^\s*#\*/ .. /\s*#\*\*/ }
    close( SCRIPT );

    optionUsage( {}, $license );
    return 1;
}

#-----------------------------------------------------------------------------

sub getMandatoryValue
{
    my $option = shift @ARGV;
    if( @ARGV == 0 or $ARGV[0] =~ /^-[^0-9]/ ) {
	local $\ = "\n";
        print( STDERR "missing argument to option $option for $0");
	die;
    }
    my $value = substr($ARGV[0], 0, 1) ne "@" ?
	$ARGV[0] : scalar(&interpolateFile( $ARGV[0], $option ));
    checkOptionType( $option, $value, $Types{$option} );
    return $value;
}

sub getOptionalValue
{
    if( @ARGV <= 1 or $ARGV[1] =~ /^-[^0-9]/ ) { return "" }
    else { 
        my $option = shift @ARGV;
        my $value = $ARGV[0] =~ /^@/ ? 
	    $ARGV[0] : scalar(&interpolateFile( $ARGV[0], $option ));
        checkOptionType( $option, $value, $Types{$option} );
	return $value;
    }
}

#-----------------------------------------------------------------------------

sub checkOptionType
{
    my ( $option, $value, $type ) = @_;

    return unless defined $type;

    my $type_descriptor = $OptionTypes{$type};
    if( ! defined $type_descriptor ) {
        die("Incorrect option type programmed for option $option");
    }

    my ($regexp, $message) = @{$type_descriptor};
    if( $value !~ /$regexp/x ) {
        print STDERR "option $option needs $message\n";
        die;
    }
}

#-----------------------------------------------------------------------------

sub interpolateFile
{
    my ($file_name, $option) = ( substr($_[0], 1), $_[1] );
    open( VALUE, $file_name ) or do {
	print STDERR "Could not open file '$file_name': $!\n";
        print STDERR "(the file was an argument for option $option of $0)\n" 
	    if $option;
        die;
    };
    if( wantarray ) { 
	my @return = map {
               chomp($_);
	       s/ *$//;
               if( /^ *-/ ) {
                  my @file_line =
		     /^ *-{1,2}[^ =]+=/ ? split(/=/,$_, 2) : split(' ',$_, 2);
		  $file_line[1] =~ s/^([^'"]*)["']|["']([^"']*)$/$1/g 
		      if $file_line[1];
                  @file_line
	       } else { $_ } 
            }
            grep !/^ *#|^ *$/, <VALUE>;
            close(VALUE) or die("Error reading $file_name: $!");
            @return;
    } else {
        my $return = join("", <VALUE>);
        close(VALUE) or die("Error reading $file_name: $!");
        $return;
    }
}

#-----------------------------------------------------------------------------

sub guessLongOption
{
    my ($option_prefix, $options) = @_;
    my @found_options;
    foreach (keys %$options) {
        push(@found_options, $_) if /$option_prefix/;
    }
    if( @found_options > 1 ) {
        $" = ", "; $\ = "\n";
	print "Option prefix '$option_prefix' not unique for $0:";
        print "Possible options are: @found_options";
        die;	
    }
    return $found_options[0];
}

return 1;
