#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 09:40:28 +0300 (Fri, 12 May 2006) $
#$Revision: 1056 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/ImageRange.pm $
#-----------------------------------------------------------------------
# Find ranges of images with continuous phi anles
#-----------------------------------------------------------------------

package ImageRange;

use strict;
require Exporter;
@ImageRange::ISA = qw(Exporter);
@ImageRange::EXPORT = qw( findImageRanges );

sub mod360 { my $x = shift;  $x - int($x) + $x % 360 }
sub round2 { my $number = $_[0]; int($number*100+0.5)/100 }

sub findImageRanges
{
    my $headers = shift;
    my $user_delta_phi = shift;

    #--------------------------------------------------------------------
    # Get angles and image numbers from all specified images
    #
    
    my @headers = map { 
    	$_->{name} =~ /_([0-9]{3})/;
    	my ($ident, $number) = ( $`, $1 );
    	print STDERR "Image names must have form 'name_001.image'\n" and die
    	    unless defined $ident or not defined $number;
    	{ ident => $ident, number => int($number), %$_ }
    } @$headers;
    
    @headers = sort {$a->{number} <=> $b->{number}} @headers;
    
    #-----------------------------------------------------------------------
    # Split image list into images runs with continuously increasing angles
    #
    
    my @runs = ();
    my @current_run = ();
    my $delta_phi;
    
    while (@headers) {
    	if( @current_run and
    	    round2(mod360($headers[0]{phi_end} - $current_run[-1]{phi_end}))
    		== $delta_phi and
            ( $headers[0]{software_id} eq "" or
              round2(mod360($headers[0]{phi_end} - $headers[0]{phi_start}))
                == $delta_phi
            )
          ) {
            $headers[0]{delta_phi} = $delta_phi;
    	    push(@current_run, shift(@headers));
    	} else {
    	    push(@runs, [@current_run]) if @current_run;
            if( defined $user_delta_phi and $user_delta_phi ne "" ) {
		$delta_phi = $user_delta_phi;
            } elsif( $headers[0]{software_id} ) {
		$delta_phi = round2(mod360($headers[0]{phi_end} - $headers[0]{phi_start}));
            } elsif( @headers >= 2 ) {
		$delta_phi = round2(mod360($headers[1]{phi_end} - $headers[0]{phi_end}));
            } else {
                die("cannot find delta_phi")
            }
            $headers[0]{delta_phi} = $delta_phi;
            $headers[1]{delta_phi} = $delta_phi if @headers > 1;
    	    @current_run = splice(@headers,0,2);
    	}
    }
    
    push(@runs, [@current_run]) if @current_run;
    return @runs;
}
