#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2008-12-29 17:55:51 +0200 (Mon, 29 Dec 2008) $
#$Revision: 1216 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/MSC.pm $
#-----------------------------------------------------------------------
# Reading header information from Mar IPS files
#-----------------------------------------------------------------------

package MSC;

use strict;
require Exporter;
@MSC::ISA = qw(Exporter);
@MSC::EXPORT = qw( readRaxisIVHeader
                   readHeader
                   readHashHeaders
                   readArrayHeaders );

sub readHeader
{
    my $filename = $_[0];
    my @retval = ();

    eval{ @retval = &readRaxisIVHeader };
    return @retval unless $@;

    die("file '$filename' is not an Raxis .osc image: $@");
}

#------------------------------------------------------------------------------

# @MSC::header_fields = qw(
# );

sub readRaxisIVHeader
{
    my ( $filename, $osc_header ) = @_;
    my $bin_header = "";
    my $header_size = 1500; # Apparently, the minimum header size...

    open(OSCFILE, $filename) or die("Can't open $filename: $!");
    read(OSCFILE, $bin_header, $header_size);
    close(OSCFILE);

    if( length($bin_header) != $header_size ) {
        close OSCFILE;
        die("too short\n");
    }

    my @osc_header = unpack( "N*", $bin_header );

    my $instrument = unpack("a10", $bin_header);

    my $image_date = unpack("x256a12", $bin_header);

    my $pixels_x = $osc_header[192];
    my $pixels_y = $osc_header[193];

    my $pixel_size_x = float_from_int( $osc_header[194] );
    my $pixel_size_y = float_from_int( $osc_header[195] );

    # The following values will need conversion to float:

    my $wavelength = float_from_int( $osc_header[73] );
    my $distance   = float_from_int( $osc_header[86] );
    my $phi_start  = float_from_int( $osc_header[131] );
    my $phi_end    = float_from_int( $osc_header[132] );
    my $exp_time   = float_from_int( $osc_header[134] );
    my $center_x   = float_from_int( $osc_header[135] );
    my $center_y   = float_from_int( $osc_header[136] );

    my $raster_offset = $pixels_x * 2;

    my %osc_header = (
        image_format => "Rigaku MSC .osc",
	instrument   => $instrument,
        ## byte_swap    => "",
        ## r_min        => $r_min,
        ## r_max        => $r_max,
        pixels_x     => $pixels_x,
        pixels_y     => $pixels_y,
        center_x     => $center_x,
        center_y     => $center_y,

        pixel_size_x => $pixel_size_x,
        pixel_size_y => $pixel_size_x,

        wavelength   => $wavelength,
        lambda       => $wavelength,
        distance     => $distance,
        ## resolution   => $resolution,
        phi_start    => $phi_start,
        phi_end      => $phi_end,
        image_date   => $image_date,

        counts_start  => 0,
        counts_end    => $exp_time,
        ## exposure_time => $osc_header[8],
        ## exposure_dose => $osc_header[9],

	raster_offset => $raster_offset,

        ## overflow_value_offset => "",
	overflow_pixels       => 0,
	## overflow_multiplier   => $osc_header[24],

        ## software_id    => $osc_strings[1],
        ## scanner_id     => $osc_strings[2],
        ## gain           => 1.0,
        ## roff           => 0.0,
	## toff           => 0.0,
        ## omega_position => $osc_header[23],
        ## omega_start    => 0.0,
        ## omega_end      => 0.0,
        ## chi            => 0.0,
	## two_theta      => 0.0,
        ## monochromator  => "",
        ## polarisation   => "",
    );

    return %osc_header if not defined $osc_header;

    if( ref $osc_header eq "ARRAY" ) {
        @$osc_header = %osc_header;
    } elsif( ref $osc_header eq "HASH" ) {
        %$osc_header = %osc_header;
    } else {
        die("Need array or hash ref as a second argument for readHeader");
    }
}

sub float_from_int
{
    my $int = $_[0];
    return unpack( "f", pack( "L", $int ));
}

return 1;
