#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2009-01-23 09:25:36 +0200 (Fri, 23 Jan 2009) $
#$Revision: 1218 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/MTZ.pm $
#-----------------------------------------------------------------------
# Reading header information from CCP4 .mtz format reflection files
#-----------------------------------------------------------------------

package MTZ;

use strict;
require Exporter;
@MTZ::ISA = qw(Exporter);
@MTZ::EXPORT = qw( readMTZHeader readHashHeaders );

sub readMTZHeader
{
    my ( $mtz_file, $mtz_header, $header_keys ) = @_;

    my $bin_header = "";
    open(MTZ, $mtz_file) or die("Could not open file '$mtz_file': $!");
    read(MTZ, $bin_header, 8) or die("Error reading file $mtz_file: $!");
    
    my ($signature, $offset) = unpack("a4I", $bin_header);

    die("File '$mtz_file' is not an MTZ file") unless $signature eq 'MTZ ';    
    seek(MTZ, ($offset - 1) * 4, 0) or die("Error seeking file $mtz_file: $!");
    $bin_header = <MTZ>;
    
    my %header = ();
    my @header_keys = ();
    my @columns = ();
    while( length($bin_header) > 0 ) {
    	my @line = split(' ', substr($bin_header, 0, 80), 2);
    	substr($bin_header, 0) = substr($bin_header, 80);
    	my $key = $line[0];
        if( ! defined $header{$key} ) {
	    $header{$key} = $line[1];
	    push( @header_keys, $key );
	} else {
            if( ref($header{$key}) eq "ARRAY" ) {
    	        push( @{$header{$key}}, $line[1] );
            } else {
                $header{$key} = [ $header{$key}, $line[1] ];
            }
        }
    	push( @columns, $line[1] ) if $key eq "COLUMN";
        SWITCH: for( $key ) {
            if(/CELL/) {
                $header{cell} = [ split(" ", $line[1]) ];
                last
	    }
            if(/RESO/) {
		$header{resolution} = [ map {1/sqrt($_)} split(" ", $line[1]) ];
		last 
	    }
            if(/SYMINF/) {
		my @line1sq;
		my $sg;
		if( $line[1] =~ /'/ ) {
		    @line1sq = split( "'", $line[1] );
		    $sg = $line1sq[1];
		} else {
		    @line1sq = split( " ", $line[1] );
		    $sg = $line1sq[4];
		}
		my @line1 = split(" ", $line[1]);
		my $sg_nosp = $sg;
		$sg_nosp =~ s/\s//g;
		$header{symmetry_data} = [ @line1[0..3], $sg_nosp, $line1[-1] ];
		$header{spacegroup} = $sg;
                last
	    }
            if(/NCOL/) {
		( $header{column_number},
		  $header{record_number} ) = split(" ", $line[1]);
                last
	    }
            if(/COLUMN/) {
		my @column = split(" ", $line[1]);
		push( @{$header{labels}}, $column[0] );
		push( @{$header{types}},  $column[1] );
		push( @{$header{min}},    $column[2] );
		push( @{$header{max}},    $column[3] );
                push( @{$header{column}},
		      { label => $column[0],
			type  => $column[1],
			min   => $column[2],
			max   => $column[3] });
                last
	    }
        }
    }

    $header{float_size} = 4;
    if( $header{SYMM} ) {
	$header{SYMM} = [ $header{SYMM} ] if ref($header{SYMM} ne "ARRAY");
	$header{symop} = $header{SYMM};
    }

    ## Return header information in the required format

    return %header if not defined $mtz_header;

    if( ref $mtz_header eq "HASH" ) {
        %$mtz_header = %header;
        @$header_keys = @header_keys if ref($header_keys) eq "ARRAY";
    } else {
        die("Need hash ref as a second argument for readMTZHeader");
    }
}

sub readHashHeaders
{
    my %header;
    my @ret = ();
    foreach (@_) {
        readMTZHeader($_, \%header);
        $header{name} = $_;
        push( @ret, { %header } );
    }
    @ret;
}
