#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2008-12-29 16:48:12 +0200 (Mon, 29 Dec 2008) $
#$Revision: 1212 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/Mar.pm $
#-----------------------------------------------------------------------
# Reading header information from Mar IPS files
#-----------------------------------------------------------------------

package Mar;

use strict;
require Exporter;
@Mar::ISA = qw(Exporter);
@Mar::EXPORT = qw( readImageHeader
                   readMar345Header
                   readMarCCD
                   readHeader readVaxHeader
                   readHashHeaders
                   readArrayHeaders );

sub readVaxHeader { &readHeader }

sub readHeader
{
    my $filename = $_[0];
    my @retval = ();

    eval{ @retval = &readImageHeader };
    return @retval unless $@;
    eval{ @retval = &readMar345Header };
    return @retval unless $@;

    die("file '$filename' is not a MAR image: $@");
}

#------------------------------------------------------------------------------

# @Mar::header_fields = qw(
# );

my $sizeof_integer = 4;
my $sizeof_float = 4;
my $binary_field_size = $sizeof_integer * 10 + 
                        $sizeof_float * 15;

my $string_field_size =  29 * 16;

sub readImageHeader
{
    my ( $filename, $mar_header ) = @_;
    my $bin_header = "";
    my $header_size = $binary_field_size + $string_field_size;

    open(MARFILE, $filename) or die("Can't open $filename: $!");
    read(MARFILE, $bin_header, $header_size);
    close(MARFILE);

    my $vax_floats = 0;
    my @mar_header = ();
    my @mar_strings = ();

    if( length($bin_header) != $header_size ) {
        close MARFILE;
        die("too short\n");
    }

    @mar_header = unpack( "V10", $bin_header );
    @mar_header = unpack( "N10", $bin_header )
        unless $mar_header[0] == 1200 or $mar_header[0] == 2000 or
               $mar_header[0] == 2300 or $mar_header[0] == 3450;
    die("incorrect header\n")
        unless $mar_header[0] == 1200 or $mar_header[0] == 2000 or
               $mar_header[0] == 2300 or $mar_header[0] == 3450;

    @mar_strings = unpack("x$binary_field_size a24" . ("a16" x 29),
                          $bin_header);

    @mar_strings = map {s/\0*//g; s/^\s*|\s*$//g; $_} @mar_strings;

    my $software_signature = $mar_strings[1];
    my $computer_architecture = $mar_strings[3];

    # check if the image has been written with marControl software.
    # marControl writes a lot of usefull information to the image.  If
    # the image has been written in the VAX architecture machines,
    # floating point number conversion will be necessary.

    if( $software_signature !~ /^\s*$/ ) {
        $vax_floats = ($computer_architecture =~ "VAX");
    } else {
        $vax_floats = 1;
    }

    if( $vax_floats ) {
        push( @mar_header, unpack( "x40N15", $bin_header ));
        @mar_header[10..24] = map {vax2ieee($_)} @mar_header[10..24];
    } else {
        push( @mar_header, unpack( "x40f15", $bin_header ));
    }

    @mar_header = (@mar_header, @mar_strings);

    my $pixel_size_x = 0.15;
    my $pixel_size_y = 0.15;

    if( $software_signature ne "" && 
	   $mar_strings[18] =~ /^[\d\.]+$/ &&
	   $mar_strings[19] =~ /^[\d\.]+$/ ) {
	$pixel_size_x = $mar_strings[18];
	$pixel_size_y = $mar_strings[19];	
    }

    my $r_min = $mar_header[12] * 2 < $mar_header[0] ?
                    $mar_header[12] :       # header contains r_min in mm
                    $mar_header[12] * $pixel_size_y;
                                            # header contains r_min in pixels,
                                            # convert to mm

    my $r_max = $mar_header[11] * 2 < $mar_header[0] ?
                    $mar_header[11] :       # header contains r_max in mm
                    $mar_header[11] * $pixel_size_y;
                                            # header contains r_max in pixels,
                                            # convert to mm
    $r_min = 0 if $r_min == $r_max;

    my $pixels_x = $mar_header[0];
    my $pixels_y = $mar_header[1];
    my $wavelength = $mar_header[19];
    my $distance = $mar_header[20];
    my $theta = 0.5 * atan2($r_max, $distance);
    my $resolution = $wavelength / (2 * sin($theta));

    my %mar_header = (
        image_format => "mar300",
        byte_swap    => "",
        r_min        => $r_min,
        r_max        => $r_max, 
        pixels_x     => $pixels_x,
        pixels_y     => $pixels_y,
        center_x     => $mar_header[17] < 2 * $r_max ?
                             $mar_header[17] : $mar_header[17] * $pixel_size_x,
        center_y     => $mar_header[18] < 2 * $r_max ?
                             $mar_header[18] : $mar_header[18] * $pixel_size_y,

        pixel_size_x => $pixel_size_x,
        pixel_size_y => $pixel_size_x,

        wavelength   => $wavelength,
        lambda       => $wavelength,
        distance     => $distance,
        resolution   => $resolution,
        phi_start    => $mar_header[21],
        phi_end      => $mar_header[22],
        image_date   => $mar_strings[0],

        counts_start  => $mar_header[6],
        counts_end    => $mar_header[7],
        exposure_time => $mar_header[8],
        exposure_dose => $mar_header[9],

	raster_offset         => $mar_header[0] == 1200 ? 2400 : 4000,
        overflow_value_offset => "",
	overflow_pixels       => $mar_header[4],
	overflow_multiplier   => $mar_header[24],

        software_id    => $mar_strings[1],
        scanner_id     => $mar_strings[2],
        gain           => 1.0,
        roff           => 0.0,
	toff           => 0.0,
        omega_position => $mar_header[23],
        omega_start    => 0.0,
        omega_end      => 0.0,
        chi            => 0.0,
	two_theta      => 0.0,
        monochromator  => "",
        polarisation   => "",
    );

    return %mar_header if not defined $mar_header;

    if( ref $mar_header eq "ARRAY" ) {
        @$mar_header = %mar_header;
    } elsif( ref $mar_header eq "HASH" ) {
        %$mar_header = %mar_header;
    } else {
        die("Need array or hash ref as a second argument for readHeader");
    }
}

sub vax2ieee {
    my $float = unpack("f", pack("v2", unpack("n2", pack("L", @_))) );
    $float / 4;
}

#---------------------------------------------------------------------------


sub readMar345Header
{
    my ( $filename, $mar_header ) = @_;
    my $bin_header = "";
    my @mar_header = ();
    my $header_size = 64;

    open(MARFILE, "$filename") or die("Can't open $filename: $!");
    read(MARFILE, $bin_header, $header_size);

    if( length($bin_header) != $header_size ) {
        close MARFILE;
        die("too short\n");
    }

    @mar_header = unpack( "V16", $bin_header );
    @mar_header = unpack( "N16", $bin_header ) unless $mar_header[0] == 1234;
    die("incorrect header\n") unless $mar_header[0] == 1234;

    @mar_header[6..15] = map { ( $_ < 2**31 ? $_ : -(~$_+1)) / 1000 }
                         @mar_header[6..15];
    $mar_header[8] /= 1000;

    # read keyworded input from the header

    my %mar_header = ();

    while(1) {
	read(MARFILE, $bin_header, $header_size);
        die("too short\n") unless length($bin_header) == $header_size;
        next if $bin_header =~ /^[\s\0]*$/;
        last if $bin_header =~ /END OF HEADER/;
        $mar_header{file_id} = $bin_header and next
            if $bin_header =~ /^mar research\s*$/i;
        my ($key, $value) = split(" ", $bin_header, 2);
        $value =~ s/\0//g;
        $value =~ s/^\s*|\s*$//g;
	if( exists $mar_header{$key} ) {
	    $mar_header{$key} .= " " . $value;
	} else {
	    $mar_header{$key} = $value;
	}
    };

    close(MARFILE);

    my $number =
         '[-+]? (?:[0-9]+(?:\.[0-9]*)? | (?:\.[0-9]+)) (?:[eE][-+]?[0-9]+)?';

    $mar_header{PIXEL} =~ / LENGTH \s* ($number) \s* HEIGHT \s* ($number) /xi; 
    my ($pixel_size_x, $pixel_size_y) = ($1/1000, $2/1000);
    $mar_header{CENTER} =~ / X \s* ($number) \s* Y \s* ($number) /xi;
    my ($center_x, $center_y) = ( $1 * $pixel_size_x, $2 * $pixel_size_y );
    $mar_header{COUNTS} =~ / START \s* ($number) \s* END \s* ($number) /xi;
    my ($counts_start, $counts_end) = ($1, $2);
    $mar_header{OFFSET} =~ / ROFF \s* ($number) \s* TOFF \s* ($number) /xi;
    my ($roff, $toff) = ($1, $2);
    $mar_header{OMEGA} =~ / START \s* ($number) \s* END \s* ($number) /xi; 
    my ($omega_start, $omega_end) = ($1, $2);
    $mar_header{MONOCHROMATOR} =~ / (\w+) \s* POLAR \s* ($number) /xi;
    my ($mono, $polar) = ($1, $2);

    my $overflow_pixels = $mar_header[2];
    my $r_max = ($mar_header[1] * $pixel_size_x) / 2;
    my $distance = $mar_header[9];
    my $wavelength = $mar_header[8];
    my $theta = 0.5 * atan2($r_max, $distance);
    my $resolution = $wavelength / (2 * sin($theta));

    %mar_header = ( 
        image_format => "mar345",
        byte_swap    => "",
        r_min        => 0,
        r_max        => $r_max,
        pixels_x     => $mar_header[1],
        pixels_y     => $mar_header[1],
        center_x     => $center_x,
        center_y     => $center_y,
        pixel_size_x => $pixel_size_x,
        pixel_size_y => $pixel_size_y,

        wavelength   => $wavelength,
        lambda       => $wavelength,
        distance     => $distance,
        resolution   => $resolution,
        phi_start    => $mar_header[10],
        phi_end      => $mar_header[11],
        image_date   => $mar_header{DATE},

        counts_start  => $counts_start,
        counts_end    => $counts_end,
        exposure_time => $mar_header{TIME},
        exposure_dose => $counts_start - $counts_end, # ???

	raster_offset         => 4096 + int($overflow_pixels/8.0 + 7.0/8.0) * 512,
        overflow_value_offset => "",
	overflow_pixels       => $overflow_pixels,
	overflow_multiplier   => $mar_header{MULTIPLIER},

        software_id    => $mar_header{PROGRAM},
        scanner_id     => $mar_header{SCANNER},
        gain           => $mar_header{GAIN},
        roff           => $roff,
	toff           => $toff,
        omega_position => $omega_start,
        omega_start    => $omega_start,
        omega_end      => $omega_end,
        chi            => $mar_header{CHI},
	two_theta      => $mar_header{TWOTHETA},
        monochromator  => $mono,
        polarisation   => $polar,
        %mar_header,
    );

    return %mar_header if not defined $mar_header;

    if( ref $mar_header eq "ARRAY" ) {
        @$mar_header = %mar_header;
    } elsif( ref $mar_header eq "HASH" ) {
        %$mar_header = %mar_header;
    } else {
        die("Need array or hash ref as a second argument for readHeader");
    }
}

sub readMarCCDHeader
{
}

sub readHashHeaders
{
    my %header;
    my @ret = ();
    foreach (@_) {
        readHeader($_, \%header);
        $header{name} = $_;
        push( @ret, { %header } );
    }
    @ret;
}

sub readArrayHeaders
{
    my @header;
    my @ret = ();
    foreach (@_) {
        readHeader($_, \@header);
        unshift( @header, $_ );
        push( @ret, [ @header ] );
    }
    @ret;
}

1;
