#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 09:40:28 +0300 (Fri, 12 May 2006) $
#$Revision: 1056 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/MarSgi.pm $
#-----------------------------------------------------------------------
# Reading header information from Mar IPS files
#-----------------------------------------------------------------------

package Mar;

use strict;
require Exporter;
@Mar::ISA = qw(Exporter);
@Mar::EXPORT = qw( readVaxHeader readSgiHeader readHashHeaders readArrayHeaders );

@Mar::header_fields = qw(
    total_pixels_x
    total_pixels_y
    lrecl
    max_rec
    overflow_pixels
    overflow_records
    counts_per_sec_start
    counts_per_sec_end
    exposure_time_sec
    programmed_exp_time_units
    programmed_exposure_time
    r_min
    r_max
    p_r
    p_l
    p_x
    p_y
    centre_x
    centre_y
    lambda
    distance
    phi_start
    phi_end
    omega
    multiplier
    scanning_date_time
);
#    filler[(600-31)*sizeof(int)]


my $sizeof_integer = 4;
my $sizeof_float = 4;
my $integer_header_size = $sizeof_integer * 10;
my $rest_header_size = $sizeof_float * 15 + 24;

sub readVaxHeader
{
    my ( $filename, $mar_header ) = @_;
    my $bin_header = "";
    my @mar_header = ();

    open(MARFILE, $filename) or die("Can't open $filename: $!");

    read(MARFILE, $bin_header, $integer_header_size);
    @mar_header = unpack( "V10", $bin_header );
    if( !defined $mar_header[0] or ($mar_header[0] != 1200 
                                and $mar_header[0] != 2000) ) {
        close MARFILE;
        die("file '$filename' is not a MAR image file\n");
    }

    read(MARFILE, $bin_header, $rest_header_size);
    push( @mar_header, unpack( "N15A24", $bin_header ));
    @mar_header[10..24] = map {vax2ieee($_)} @mar_header[10..24];

    close(MARFILE);

    return @mar_header if not defined $mar_header;

    if( ref $mar_header eq "ARRAY" ) {
        @$mar_header = @mar_header;
    } elsif( ref $mar_header eq "HASH" ) {
        my $field = 0;
        %$mar_header = map { $Mar::header_fields[$field++], $_ } @mar_header;
    } else {
        die("Need array or hash ref as a second argument for readVaxHeader");
    }
}

sub vax2ieee {
    my $float = unpack("f", pack("v2", unpack("n2", pack("L", @_))) );
    $float / 4;
}

sub readSgiHeader
{
    my ( $filename, $mar_header ) = @_;
    my $bin_header = "";
    my @mar_header = ();

    open(MARFILE, $filename) or die("Can't open $filename: $!");

    read(MARFILE, $bin_header, $integer_header_size);
    @mar_header = unpack( "I10", $bin_header );
    if( !defined $mar_header[1] or ($mar_header[1] != 1200 
                                and $mar_header[1] != 2000) ) {
        close MARFILE;
        die("file '$filename' is not a MAR image file\n");
    }

    read(MARFILE, $bin_header, $rest_header_size);
    push( @mar_header, unpack( "f15A24", $bin_header ));
##    @mar_header[10..24] = map {vax2ieee($_)} @mar_header[10..24];

    close(MARFILE);

    return @mar_header if not defined $mar_header;

    if( ref $mar_header eq "ARRAY" ) {
        @$mar_header = @mar_header;
    } elsif( ref $mar_header eq "HASH" ) {
        my $field = 0;
        %$mar_header = map { $Mar::header_fields[$field++], $_ } @mar_header;
    } else {
        die("Need array or hash ref as a second argument for readVaxHeader");
    }
}

sub readHashHeaders
{
    &readHashSgiHeaders;
}

sub readHashVaxHeaders
{
    my %header;
    my @ret = ();
    foreach (@_) {
        readVaxHeader($_, \%header);
        $header{name} = $_;
        push( @ret, { %header } );
    }
    @ret;
}

sub readHashSgiHeaders
{
    my %header;
    my @ret = ();
    foreach (@_) {
        readSgiHeader($_, \%header);
        $header{name} = $_;
        push( @ret, { %header } );
    }
    @ret;
}

sub readArrayHeaders
{
    my @header;
    my @ret = ();
    foreach (@_) {
        readVaxHeader($_, \@header);
        unshift( @header, $_ );
        push( @ret, [ @header ] );
    }
    @ret;
}
