#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 09:40:28 +0300 (Fri, 12 May 2006) $
#$Revision: 1056 $
#$URL: svn+ssh://lokys.ibt.lt/home/saulius/svn-repositories/cce/modules/SymopAlg.pm $
#-----------------------------------------------------------------------
#*
# Basic symmetry operator algebra (addition, multiplication, etc.)
#**

package MatrixAlg;

use strict;

require Exporter;
@MatrixAlg::ISA = qw(Exporter);
@MatrixAlg::EXPORT = qw(
    matrix_transpose
    matrix_times_vector
    matrix3x3_invert
    matrix3x3_det
);

sub matrix3x3_det( $ )
{
    my $m = $_[0];
    return
        + $m->[0][0] * $m->[1][1] * $m->[2][2]
        + $m->[1][0] * $m->[2][1] * $m->[0][2]
        + $m->[0][1] * $m->[1][2] * $m->[2][0]

        - $m->[0][2] * $m->[1][1] * $m->[2][0]
        - $m->[0][0] * $m->[1][2] * $m->[2][1]
        - $m->[0][1] * $m->[1][0] * $m->[2][2];
}

sub matrix2x2_det($)
{
    my $m = $_[0];
    return $m->[0][0]*$m->[1][1] - $m->[0][1]*$m->[1][0];
}

sub matrix3x3_adjunct($$$)
{
    my ( $m, $row, $col ) = @_;
 
    my @matrix = ();
    my ( $i, $j, $mi, $mj );
    my $coef;

    die unless( $row >= 0 && $row < 3 );
    die unless( $col >= 0 && $col < 3 );

    $mi = $mj = 0;
    for( $i = 0; $i < 3; $i ++ ) {
        next if( $i == $row );
        $mj = 0;
        for( $j = 0; $j < 3; $j ++ ) {
	    next if( $j == $col );
            $matrix[$mi][$mj] = $m->[$i][$j];
            $mj ++;
        }
        $mi ++;
    }
    die unless( $mi == 2 );
    die unless( $mj == 2 );
    $coef = (($row + $col) % 2 == 0) ? +1.0 : -1.0;
    return $coef * matrix2x2_det( \@matrix );
}

sub matrix3x3_invert( $ )
{
    my $m = $_[0];
    my @ret;
    my $det = matrix3x3_det( $m );
    my $N = @{$m};

    my ( $i, $j );
    for( $i = 0; $i < $N; $i++ ) {
        for( $j = 0; $j < $N; $j++ ) {
            $ret[$i][$j] = matrix3x3_adjunct($m,$j,$i) / $det;
        }
    }

    return \@ret;
}

sub matrix_transpose( $ )
{
    my $m = $_[0];

    for my $i (1..$#{$m}) {
        for my $j (0..$i-1) {
            my $tmp = $m->[$i][$j];
            $m->[$i][$j] = $m->[$j][$i];
            $m->[$j][$i] = $tmp;
        }
    }

    return $m;
}

sub matrix_times_vector( $$ )
{
    my ($m, $v) = @_;

    my @res;

    for my $i (0..$#$m) {
        $res[$i] = 0.0;
        for my $j (0..$#$v) {
            $res[$i] += $m->[$i][$j] * $v->[$j];
        }
    }

    return \@res;
}

1;
