#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 13:42:34 +0300 (Fri, 12 May 2006) $
#$Revision: 1064 $
#$URL: svn+ssh://lokys.ibt.lt/home/saulius/svn-repositories/cce/modules/PDB.pm $
#-----------------------------------------------------------------------
#
# Align two sequences using Needleman-Wunsch algorithm, using affine
# gap penalty.
#
#-----------------------------------------------------------------------

package NWAffine;

use strict;
require Exporter;
@NWAffine::ISA = qw(Exporter);
@NWAffine::EXPORT_OK = qw( align );

$NWAffine::debug = 0;

sub align {
    my ( $seq1, $seq2, $S, $gap_open, $gap_extend ) = @_;

    $gap_open = -1 if !defined $gap_open;
    $gap_extend = -0.5 if !defined $gap_extend;

    my @seq1 = split( "", $seq1 );
    my @seq2 = split( "", $seq2 );

    print join( ":", @seq1 ), "\n" if $NWAffine::debug & 1;
    print join( ":", @seq2 ), "\n" if $NWAffine::debug & 1;

    ## my @F = map{ [ {} x (int(@seq2)+1)] } ("",@seq1);

    my @F;

    for my $i (0..$#seq1+1) {
        for my $j (0..$#seq2+1) {
            $F[$i][$j] = {};
        }
    }

    print_matrix( @F ) if $NWAffine::debug & 2;

    for my $i (0..$#seq1+1) {
	$F[$i][0]{M} = $gap_open * $i;
	$F[$i][0]{Ix} = $gap_open * $i;
	$F[$i][0]{Iy} = $gap_open * $i;
    }
    for my $j (0..$#seq2+1) {
	$F[0][$j]{M} = $gap_open * $j;	
	$F[0][$j]{Ix} = $gap_open * $j;	
	$F[0][$j]{Iy} = $gap_open * $j;	
    }
    for my $i (1..$#seq1+1) {
	for my $j (1..$#seq2+1) {
            my $subst_score = Score( $seq1[$i-1], $seq2[$j-1], $S );
	    my $Mchoice1 = $F[$i-1][$j-1]{M} + $subst_score;
	    my $Mchoice2 = $F[$i-1][$j-1]{Ix} + $subst_score;
	    my $Mchoice3 = $F[$i-1][$j-1]{Iy} + $subst_score;

            $F[$i][$j]{M} = max( $Mchoice1, max( $Mchoice2, $Mchoice3 ));

	    my $Ix_choice1 = $F[$i-1][$j]{M} + $gap_open;
	    my $Ix_choice2 = $F[$i-1][$j]{Ix} + $gap_extend;

            $F[$i][$j]{Ix} = max( $Ix_choice1, $Ix_choice2 );

	    my $Iy_choice1 = $F[$i][$j-1]{M} + $gap_open;
	    my $Iy_choice2 = $F[$i][$j-1]{Iy} + $gap_extend;

            $F[$i][$j]{Iy} = max( $Iy_choice1, $Iy_choice2 );
        }
    }

    print_matrix( @F ) if $NWAffine::debug & 4;

    my $alignment1 = "";
    my $alignment2 = "";

    my ($i, $j) = ( $#seq1+1, $#seq2+1 );

    while( $i > 0 && $j > 0 ) {
	my $score_diag = $F[$i][$j]{M};
	my $score_up = $F[$i][$j]{Iy};
	my $score_left = $F[$i][$j]{Ix};

	if( $score_diag > $score_left && $score_diag > $score_up ) {
	    $alignment1 = $seq1[$i-1] . $alignment1;
	    $alignment2 = $seq2[$j-1] . $alignment2;
	    $i --;
	    $j --;
	} elsif ( $score_left > $score_diag && $score_left > $score_up ) {
	    $alignment1 = $seq1[$i-1] . $alignment1;
	    $alignment2 = "-" . $alignment2;
	    $i --;
	} else {
	    $alignment1 = "-" . $alignment1;
	    $alignment2 = $seq2[$j-1] . $alignment2;
	    $j --;
	}

	do {
	    print ">>> \$score_diag = $score_diag\n";
	    print ">>> \$score_up   = $score_up\n";
	    print ">>> \$score_left = $score_left\n";
	    print ">>> \$i = $i, \$j = $j\n";
	    print ">>> \$alignment1 = $alignment1\n";
	    print ">>> \$alignment2 = $alignment2\n";
	    print "\n";
	} if $NWAffine::debug & 8;
    }

    while( $i > 0 ) {
	$alignment1 = $seq1[$i-1] . $alignment1;
	$alignment2 = "-" . $alignment2;
	$i --;
    }
    while( $j > 0 ) {
	$alignment1 = "-" . $alignment1;
	$alignment2 = $seq2[$j-1] . $alignment2;
	$j --;
    }

    return wantarray ?
	($alignment1, $alignment2) :
	$alignment1 . "\n" . $alignment2;
}

sub Score
{
    my ( $A, $B, $S ) = @_;

    if( defined $S ) {
	return $S->{$A}{$B}
    } else {
	return $A eq $B ? 1 : 0;
    }
}

sub max { $_[0] > $_[1] ? $_[0] : $_[1] }
sub min { $_[0] < $_[1] ? $_[0] : $_[1] }

sub print_matrix
{
    my @F = @_;

    my @S = ();

    for my $i (0..$#F) {
	for my $j (0..$#{$F[$i]}) {
	    $S[$j] .= sprintf "%3s ", $F[$i][$j];
	}
    }
    
    for my $line (@S) {
	print "$line\n";
    }
    print "\n";
}

1;
