#--*-perl-*-------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 13:42:34 +0300 (Fri, 12 May 2006) $
#$Revision: 1064 $
#$URL: svn+ssh://lokys.ibt.lt/home/saulius/svn-repositories/cce/modules/PDB.pm $
#-----------------------------------------------------------------------
#
# Align two sequences using Needleman-Wunsch algorithm.
#
#-----------------------------------------------------------------------

package NWAlign;

use strict;
require Exporter;
@NWAlign::ISA = qw(Exporter);
@NWAlign::EXPORT_OK = qw( align );

$NWAlign::debug = 0;

sub align {
    my ( $seq1, $seq2, $S, $gap ) = @_;

    $gap = -1 if !defined $gap;

    my @seq1 = split( "", $seq1 );
    my @seq2 = split( "", $seq2 );

    print join( ":", @seq1 ), "\n" if $NWAlign::debug & 1;
    print join( ":", @seq2 ), "\n" if $NWAlign::debug & 1;

    my @F = map{ [(0) x (int(@seq2)+1)] } ("",@seq1);

    print_matrix( @F ) if $NWAlign::debug & 2;

    for my $i (0..$#seq1+1) {
	$F[$i][0] = $gap * $i;
	
    }
    for my $j (0..$#seq2+1) {
	$F[0][$j] = $gap * $j;
	
    }
    for my $i (1..$#seq1+1) {
	for my $j (1..$#seq2+1) {
	    my $choice1 = $F[$i-1][$j-1] + Score( $seq1[$i-1],$seq2[$j-1],$S );
	    my $choice2 = $F[$i-1][$j] + $gap;
	    my $choice3 = $F[$i][$j-1] + $gap;
	    $F[$i][$j] = max( $choice1, max( $choice2, $choice3 ));
	}
    }

    print_matrix( @F ) if $NWAlign::debug & 4;

    my $alignment1 = "";
    my $alignment2 = "";

    my ($i, $j) = ( $#seq1+1, $#seq2+1 );

    while( $i > 0 && $j > 0 ) {
	my $score = $F[$i][$j];
	my $score_diag = $F[$i-1][$j-1];
	my $score_up = $F[$i][$j-1];
	my $score_left = $F[$i-1][$j];

	if( $score == $score_diag + Score( $seq1[$i-1], $seq2[$j-1], $S )) {
	    $alignment1 = $seq1[$i-1] . $alignment1;
	    $alignment2 = $seq2[$j-1] . $alignment2;
	    $i --;
	    $j --;
	} elsif ( $score == $score_left + $gap ) {
	    $alignment1 = $seq1[$i-1] . $alignment1;
	    $alignment2 = "-" . $alignment2;
	    $i --;
	} elsif ( $score == $score_up + $gap ) {
	    $alignment1 = "-" . $alignment1;
	    $alignment2 = $seq2[$j-1] . $alignment2;
	    $j --;
	} else {
	    die "Could not determine where we came from!\n"
	}

	do {
	    print ">>> \$score      = $score\n";
	    print ">>> \$score_diag = $score_diag\n";
	    print ">>> \$score_up   = $score_up\n";
	    print ">>> \$score_left = $score_left\n";
	    print ">>> \$i = $i, \$j = $j\n";
	    print ">>> \$alignment1 = $alignment1\n";
	    print ">>> \$alignment2 = $alignment2\n";
	    print "\n";
	} if $NWAlign::debug & 8;
    }

    while( $i > 0 ) {
	$alignment1 = $seq1[$i-1] . $alignment1;
	$alignment2 = "-" . $alignment2;
	$i --;
    }
    while( $j > 0 ) {
	$alignment1 = "-" . $alignment1;
	$alignment2 = $seq2[$j-1] . $alignment2;
	$j --;
    }

    return wantarray ?
	($alignment1, $alignment2) :
	$alignment1 . "\n" . $alignment2;
}

sub Score
{
    my ( $A, $B, $S ) = @_;

    if( defined $S ) {
	return $S->{$A}{$B}
    } else {
	return $A eq $B ? 1 : 0;
    }
}

sub max { $_[0] > $_[1] ? $_[0] : $_[1] }
sub min { $_[0] < $_[1] ? $_[0] : $_[1] }

sub print_matrix
{
    my @F = @_;

    my @S = ();

    for my $i (0..$#F) {
	for my $j (0..$#{$F[$i]}) {
	    $S[$j] .= sprintf "%3s ", $F[$i][$j];
	}
    }
    
    for my $line (@S) {
	print "$line\n";
    }
    print "\n";
}

1;
