#------------------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-03 13:16:28 +0300 (Wed, 03 May 2006) $ 
#$Revision: 988 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/aux/Namer.pm $
#------------------------------------------------------------------------

package File::Namer;

use strict;
require Exporter;
@File::Namer::ISA = qw( Exporter );
@File::Namer::EXPORT = qw( basename corename dirname );
@File::Namer::EXPORT_OK = 
    qw( parse base basename core corename dir dirname ext );

#
#  Split file name into directory, name and extension parts.
#

$File::Namer::pattern = '^(.*\/)*([^\/]*?)(\.[^.\/]*)?$';

sub parse
{
    $_[0] =~ /$File::Namer::pattern/;
    my $directory  = $1 ? $1 : "./";
    my $basename   = $2 ? $2 : ($3 ? $3 : "");
    my $extension  = $3 ? ($2 ? $3 : "") : "";
    return ($directory, $basename, $extension);
}

#
#  Remove directotry part and one extension from the file name
#

sub corename { &core } 

sub core
{
    $_[0] =~ /$File::Namer::pattern/;
    return $2 ? $2 : ($3 ? $3 : "");
}

#
# Return the basename of the file, as in unix 'basename' utility:
# Remove directotry part and the specified extensions from the file
# name
#

sub basename { &base }

sub base
{
    my $basename = shift;
    $basename =~ s|^.*/||;
    while (@_) { $basename =~ s/$_[0]$//; shift }
    
    return $basename;
}

#
# Return extension of the file
#

sub ext
{
    $_[0] =~ /$File::Namer::pattern/;
    my $extension  = $3 ? ($2 ? $3 : "") : "";
    return $extension;
}

#
# Return the directory part of the file name
#

sub dir
{
    $_[0] =~ /$File::Namer::pattern/;
    my $dir =  $1 ? $1 : "./";
    return $dir;
}

#
# Return the directory part of the file name, without the trailing "/"
# character (except for the root directory name, "/")  
#

sub dirname 
{ 
    my $dirname = &dir;
    $dirname =~ s|/$|| if $dirname ne "/";
    return $dirname;
}

#
#  Remove one extension from the file name, leaving directory part intact
#

sub removeExtension
{
   my $basename = $_[0];
   $basename =~ s/\.[^.\/]*$//;
   return $basename;
}

#
#  Remove all extensions from a file name, leave directory part intact
#

sub removeAllExtensions
{
   my $basename = $_[0];
   $basename =~ s/\.[^\/]*$//;
   return $basename;
}

return 1;
