#--*-perl-*-------------------------------------------------------------
#  Rotate a pdb file
#-----------------------------------------------------------------------
#$Author: saulius $
#$Date: 2013-01-16 13:39:14 +0200 (Wed, 16 Jan 2013) $
#$Revision: 1348 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/PDB.pm $
#-----------------------------------------------------------------------
#
#INPUT:
#   One ATOM or HETATM record of the PDB file as a string (read directly
#   from the PDB file, e.g. with while(<>)... )
#
#OUTPUT:
#   An array containing PDB file fields splitted into separate elements
#
#-----------------------------------------------------------------------

package PDB;

use strict;
require Exporter;
@PDB::ISA = qw(Exporter);
@PDB::EXPORT = qw( split_pdb split_atom split_anisou
		   split_sigatm slit_siguij
		   print_pdb );

$PDB::atom_format =
   "%-6s%5d %-4s%1s%3s %1s%4s%1s   %8.3f%8.3f%8.3f%6.2f%6.2f%6s%-4s%2s%2s\n";

$PDB::sigatm_format = $PDB::atom_format;

$PDB::format = $PDB::atom_format;

$PDB::anisou_format =
   "%-6s%5d %-4s%1s%3s %1s%4s%1s %7d%7d%7d%7d%7d%7d  %-4s%2s%2s\n";

$PDB::siguij_format = $PDB::anisou_format;

sub split_pdb($)
{
    my $line = $_[0];

    if( $line =~ /^(ATOM  |HETATM|SIGATM)/ ) {
	&split_atom;
    } elsif( $line =~ /^(ANISOU|SIGUIJ)/ ) {
	&split_anisou;
    } else {
	return ( $line );
    }
}

sub split_sigatm($)
{
    &split_atom
}

sub split_siguij($)
{
    &split_anisou
}

sub split_atom($)
{
   local $_ = $_[0];
   ## local $[ = 1; # PDB fields are counted from 1, in FORTRAN-ish
   ## manner; but the assignment to $[ is now deprecated...
   substr( $_, length, 80-length ) = " " x (80-length) if length() < 80;
   return (
        substr($_,  0, 6), # 0: ATOM or HETATM
        substr($_,  6, 5), # 1: Atom serial number
        substr($_, 12, 4), # 2: Atom name
        substr($_, 16, 1), # 3: Alternate location indicator
        substr($_, 17, 3), # 4: Residue name
        
        substr($_, 21, 1), # 5: Chain identifier, e.g., A for \alpha chain
        substr($_, 22, 4), # 6: residue seq. no.
        substr($_, 26, 1), # 7: insertion code
        
        substr($_, 30, 8), # 8 : X atomic coordinate
        substr($_, 38, 8), # 9 : Y atomic coordinate
        substr($_, 46, 8), # 10: Z atomic coordinate
        substr($_, 54, 6), # 11: Occupancy
        substr($_, 60, 6), # 12: Temperature factor
        substr($_, 66, 6), # 13: Filler?

        substr($_, 72, 4), # 14: Segment ID a-la xplor
        substr($_, 76, 2), # 15: Element Symbol
        substr($_, 78, 2), # 16: Charge of the atom
    )
}

sub split_anisou($)
{
   local $_ = $_[0];
   ## local $[ = 1; # PDB fields are counted from 1, in FORTRAN-ish manner 
   substr( $_, length, 80-length ) = " " x (80-length) if length() < 80;
   return (
        substr($_,  0, 6), # 0: ANISOU
        substr($_,  6, 5), # 1: Atom serial number
        substr($_, 12, 4), # 2: Atom name
        substr($_, 16, 1), # 3: Alternate location indicator
        substr($_, 17, 3), # 4: Residue name
        
        substr($_, 21, 1), # 5: Chain identifier, e.g., A for \alpha chain
        substr($_, 22, 4), # 6: residue seq. no.
        substr($_, 26, 1), # 7: insertion code
        
        substr($_, 28, 7), # 8 : Integer  u[0][0]  U(1,1)
        substr($_, 35, 7), # 9 : Integer  u[1][1]  U(2,2)
        substr($_, 42, 7), # 10: Integer  u[2][2]  U(3,3)
        substr($_, 49, 7), # 11: Integer  u[0][1]  U(1,2)
        substr($_, 56, 7), # 12: Integer  u[0][2]  U(1,3)
        substr($_, 63, 7), # 13: Integer  u[1][2]  U(2,3)

        substr($_, 72, 4), # 14: Segment ID a-la xplor
        substr($_, 76, 2), # 15: Element Symbol
        substr($_, 78, 2), # 16: Charge of the atom
    )
}

sub print_pdb(@)
{
    my @record = @_;

    if( $record[0]  =~ /^(ATOM  |HETATM|SIGATM)/ ) {
	printf $PDB::atom_format, @record;
    } elsif( $record[0] =~ /^(ANISOU|SIGUIJ)/ ) {
	printf $PDB::anisou_format, @record;
    } else {
	printf "%s\n", join( "", @record );
    }

}

1;
