#--*-perl-*--------------------------------------------------------------------
#$Author$
#$Date$
#$Revision$
#$URL$
#------------------------------------------------------------------------------
#  Module providing subroutine rotating vector $c around axis $u by 180 degrees.
#------------------------------------------------------------------------------

package Rotate180;

use strict;
require Exporter;
@Rotate180::ISA = qw( Exporter );
@Rotate180::EXPORT = qw( rotate180 );

#
# Subroutine rotate180 rotates vector $c around other vector $u by 180
# degrees.
#
# The subroutine should get a reference to an array $c that contains
# coordinates of a vector $c (x1, x2, x3) as the first argument, and a
# unit vector $u as the second argument. It multiplicates them using
# matrix for a counterclockwise rotation by an angle of 180 degrees
# about an axis the direction of u.
#
# Subroutine returns coordinates of a new rotated vector.
#
# Matrix of the counterclockwise rotation was taken from
# http://en.wikipedia.org/wiki/Rotation_matrix (2009.04.03).
#

sub rotate180
{
    my ( $c, $u ) = @_;
    return ( ( $c->[0] * ( $u->[0] * $u->[0] + ( ( 1 - $u->[0] * $u->[0] ) * (-1) ) ) ) + 
           + ( $c->[1] * ( $u->[0] * $u->[1] * 2 ) ) +
           + ( $c->[2] * ( $u->[0] * $u->[2] * 2 ) ),
             ( $c->[0] * ( $u->[0] * $u->[1] * 2 ) ) +
           + ( $c->[1] * ( $u->[1] * $u->[1] + ( ( 1 - $u->[1] * $u->[1] ) * (-1) ) ) ) +
           + ( $c->[2] * ( $u->[1] * $u->[2] * ( 2 ) ) ),
             ( $c->[0] * ( $u->[0] * $u->[2] * ( 2 ) ) ) +
           + ( $c->[1] * ( $u->[1] * $u->[2] * ( 2 ) ) ) +
           + ( $c->[2] * ( $u->[2] * $u->[2] + ( ( 1 - $u->[2] * $u->[2] ) * (-1) ) ) )
           )
}

1;
