#--*-perl-*--------------------------------------------------------------
#$Author: saulius $
#$Date: 2006-05-12 09:40:28 +0300 (Fri, 12 May 2006) $ 
#$Revision: 1056 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/Symmetry.pm $
#-----------------------------------------------------------------------
# Process symmetry operators in perl scripts.
# Searches for symmetry operators and/or space group names in the 
# symmetry libtraty file
#-----------------------------------------------------------------------

package Symmetry;

use strict;
require Exporter;
@Symmetry::ISA = qw(Exporter);
@Symmetry::EXPORT = qw( find lib );

@Symmetry::fields = qw( number aunits name class symop );

#
# possible locations were to search for symmetry library file
#
BEGIN {
    $Symmetry::Id = '$Id: Symmetry.pm 1056 2006-05-12 06:40:28Z saulius $';

    @Symmetry::libLocations = (
    	"$ENV{HOME}/lib/sym.lib",
    	"/xray/lib/sym.lib",
    	"/usr/local/lib/sym.lib",
        "./sym.lib",
    );

    unshift( @Symmetry::libLocations, "$ENV{PERL_LIB}/sym.lib" )
         if $ENV{PERL_LIB};

    foreach ( @Symmetry::libLocations ) {
	$Symmetry::lib = $_ and last if -f $_ and -r $_
    }
    if( ! $Symmetry::lib ) {
	local $" = "\n   ";
	print STDERR "\n$Symmetry::Id:\n";
        print STDERR "Could not find readable symmetry library file\n";
	print STDERR "Tried for:\n   @Symmetry::libLocations\n\n";
	die;
    }
}

sub find
{
    my $symmetry = shift;
    my $hash = shift;

    $symmetry =~ s/[^-\/A-Za-z0-9]//g;
    open(LIB, $Symmetry::lib) or
	die("Could not open symmetry library '$Symmetry::lib'");

    while(<LIB>) {
	if(/ $symmetry /i) {
	    # 5 4 2 C2         MONOCLINIC
	    my ($group_number, $symop_number, 
		$assym_units,  $group_name, 
		$crystal_class ) = split;
	    my $symops = [];
	    for (1..$symop_number) {
		my $operator = <LIB>;
		chomp $operator;
                $operator =~ s/ //g;
		push(@$symops, $operator);
	    }
	    close(LIB) or
		die("Error reading symmetry library '$Symmetry::lib'");
            if(defined $hash and ref($hash) eq "HASH" ) {
		%$hash = ( number => $group_number,
                           aunits => $assym_units,
			   name   => $group_name,
			   class  => $crystal_class,
			   symop  => $symops);
		return;
            } else {
	        return ($group_number, $symop_number,
	        	$assym_units,  $group_name, $crystal_class,
	        	$symops );
	    }
        }
    }
    close(LIB) or die("Error reading symmetry library '$Symmetry::lib'");
    die("Space group '$symmetry' not found");
}

1;
