#------------------------------------------------------------------------------
#$Author: saulius $
#$Date: 2009-10-23 15:17:16 +0300 (Fri, 23 Oct 2009) $ 
#$Revision: 1261 $
#$URL: svn+ssh://saulius-grazulis.lt/home/saulius/svn-repositories/cce/modules/UserMessageLN.pm $
#------------------------------------------------------------------------------
#*
#  Format run time error and warning messages in a uniform way,
#  reporting the calling script's name, the processed filename, and
#  the line number by which error or warning has been caused.
#**

package UserMessageLN;

use strict;
use warnings;
require Exporter;
@UserMessageLN::ISA = qw(Exporter);
@UserMessageLN::EXPORT = qw( error warning );

#==============================================================================
# Print a message, reporting a program name, file name, data block
# name and am error level (ERROR or WARNING) in a uniform way.
#
# For ease of parsing error messages from log files, $message should
# probably not contain a colon (":") since colon is used to separate
# different parts of the error message.

sub print_message($$$$$)
{
    my ( $program, $filename, $linenr, $errlevel, $message ) = @_;

    local $\ = "";
    local $, = "";

    $message =~ s/ at $0 line \d+.$//;
    $message =~ s/\n+$//;

    print STDERR $program, ": ", $filename,
    defined $linenr ? "(" . $linenr . ")" : "",
    defined $errlevel ? ": " . $errlevel : "",
    ": ", $message, $message =~ /\.$/ ? "\n" : ".\n";
}

#==============================================================================
# Report an error message. Errors are indicated with the "ERROR"
# keyword in the message line. This is supposed to be a fatal even,
# and the program will most probably die() or exit(255) after this
# message, but the UserMessageLN package does not enforce this policy.

sub error($$$$)
{
    my ( $program, $filename, $linenr, $message ) = @_;
    print_message( $program, $filename, $linenr, "ERROR", $message );
}

#==============================================================================
# Report a warning message. Warning are indicated with the "WARNING"
# keyword. Program can probably continue after warnings and give a
# reasonable result, but it might be not the result which the user
# expected.

sub warning($$$$)
{
    my ( $program, $filename, $linenr, $message ) = @_;
    print_message( $program, $filename, $linenr, "WARNING", $message );
}

1;
